import flask

template_bp = flask.Blueprint("template", __name__, url_prefix="/template")


@template_bp.route("/<template_name>/preview/")
def preview(template_name):
    return flask.render_template(f"email/{template_name}"), 200



import sqlalchemy

from affo_email_service import const
from affo_email_service.extensions import db
from affo_email_service.models.message import Message
from affo_email_service.models.unsubscribe import Unsubscribe
from affo_email_service.tasks.email import send_email


class EmailService:
    def send(
        self, from_, to, subject, text, html, tag, attachments=None, cc=None, bcc=None
    ):
        message_status = const.MESSAGE_STATUS_NEW

        with db.session.begin(subtransactions=True):
            # Ensure unsubscribes
            unsubscribes = db.session.query(Unsubscribe).filter(
                sqlalchemy.and_(Unsubscribe.email.in_(to), Unsubscribe.tag == tag)
            )

            unsubscribes = set((u.email for u in unsubscribes))

            if unsubscribes:
                message_status = const.MESSAGE_STATUS_SKIPPED

            message_to = list(set(to) - unsubscribes)

            if message_to:
                message_status = const.MESSAGE_STATUS_PARTIALLY_SKIPPED

                # Hold subscribed addresses only
                to = message_to

            # Create a message
            message_ = Message(
                from_=from_,
                to=to,
                cc=cc,
                bcc=bcc,
                subject=subject,
                text=text,
                html=html,
                attachment=bool(attachments),
                tag=tag,
                status=message_status,
            )
            db.session.add(message_)

        # Send the message
        if message_status in (
            const.MESSAGE_STATUS_NEW,
            const.MESSAGE_STATUS_PARTIALLY_SKIPPED,
        ):
            send_email.delay(message_id=message_.id)

        return message_



import sys  # noqa

from dynaconf import LazySettings, Validator

settings = LazySettings(ENVVAR_PREFIX_FOR_DYNACONF="AFFO_ES", ENVVAR_FOR_DYNACONF="AFFO_ES_SETTINGS")

# Register validators
settings.validators.register(
    Validator("DATABASE_URI", "CELERY_BROKER_URL", "EMAIL_HOST", "EMAIL_PORT", must_exist=True),
    Validator("EMAIL_PORT", is_type_of=int),
)

# Fire the validator
settings.validators.validate()

# SECRET CONFIGURATION
SECRET_KEY = getattr(settings, "SECRET_KEY", "")

# DEBUG CONFIGURATION
DEBUG = getattr(settings, "DEBUG", False)

# SQLALCHEMY CONFIGURATION
SQLALCHEMY_DATABASE_URI = settings.DATABASE_URI
SQLALCHEMY_TRACK_MODIFICATIONS = True
SQLALCHEMY_MODEL_IMPORTS = ("affo_email_service.models.message", "affo_email_service.models.unsubscribe")

# CELERY CONFIGURATION
CELERY_BROKER_URL = settings.CELERY_BROKER_URL
CELERY_IMPORTS = ("affo_email_service.tasks.email",)

# MAIL CONFIGURATION
MAIL_SERVER = settings.EMAIL_HOST
MAIL_USERNAME = getattr(settings, "EMAIL_HOST_USER", None)
MAIL_PASSWORD = getattr(settings, "EMAIL_HOST_PASSWORD", None)
MAIL_PORT = settings.EMAIL_PORT
MAIL_USE_TLS = getattr(settings, "EMAIL_USE_TLS", False)
MAIL_SUPPRESS_SEND = getattr(settings, "EMAIL_SUPPRESS_SEND", False)

settings.populate_obj(sys.modules[__name__])



from flask_script import Manager
from flask_migrate import MigrateCommand

from affo_email_service.application import create_app


app = create_app()

manager = Manager(app)
manager.add_command("db", MigrateCommand)


if __name__ == "__main__":
    manager.run()



from flask import render_template

from affo_email_service.models.message import message_schema
from affo_email_service.services import email_service


def send(template_name, message):
    message["html"] = render_template(f"email/{template_name}.html", **message["variables"])
    message["text"] = render_template(f"email/{template_name}.txt", **message["variables"])

    # FIXME: attachments aren't supported
    return (
        message_schema.jsonify(
            email_service.send(
                from_=message["from_"],
                to=message["to"],
                subject=message["subject"],
                text=message["text"],
                html=message["html"],
                tag=message["tag"],
                attachments=message.get("attachments", None),
            )
        ),
        201,
    )



import itsdangerous

from affo_email_service import settings


def token(email, tag):
    safe = itsdangerous.URLSafeTimedSerializer(settings.SECRET_KEY)

    return {"token": safe.dumps({"email": email, "tag": tag})}, 200



import datetime

from sqlalchemy_utils import ScalarListType

from affo_email_service import const
from affo_email_service.extensions import db, ma


class Message(db.Model):  # noqa
    id = db.Column(db.Integer(), primary_key=True)
    from_ = db.Column(db.String(256))
    to = db.Column(ScalarListType())
    cc = db.Column(ScalarListType(), nullable=True)
    bcc = db.Column(ScalarListType(), nullable=True)
    subject = db.Column(db.String(256))
    text = db.Column(db.Text(), nullable=True)
    html = db.Column(db.Text(), nullable=True)
    attachment = db.Column(db.Boolean(), default=False)
    tag = db.Column(db.String(64))
    status = db.Column(
        db.Enum(
            const.MESSAGE_STATUS_NEW,
            const.MESSAGE_STATUS_PENDING,
            const.MESSAGE_STATUS_FAILED,
            const.MESSAGE_STATUS_SUCCEDED,
            const.MESSAGE_STATUS_SKIPPED,
            const.MESSAGE_STATUS_PARTIALLY_SKIPPED,
        )
    )
    updated_at = db.Column(db.DateTime, default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow)
    created_at = db.Column(db.DateTime, default=datetime.datetime.utcnow)


class MessageSchema(ma.ModelSchema):
    class Meta:
        model = Message
        exclude = ("attachment",)


message_schema = MessageSchema()
messages_schema = MessageSchema(many=True)



import importlib
import logging

import connexion

import connexion_buzz

import sentry_sdk
from sentry_sdk.integrations.celery import CeleryIntegration
from sentry_sdk.integrations.flask import FlaskIntegration

from . import routes, settings, VERSION
from .extensions import celery, db, ma, mail, migrate

__all__ = ["create_celery", "create_app"]

logging.basicConfig(level=logging.INFO)


def create_celery(app, settings_override=None):
    if settings_override:
        app.config.update(settings_override)

    celery.config_from_object(app.config, namespace="CELERY")

    TaskBase = celery.Task

    class ContextTask(TaskBase):  # noqa
        abstract = True

        def __call__(self, *args, **kwargs):
            with app.app_context():
                return TaskBase.__call__(self, *args, **kwargs)

    celery.Task = ContextTask

    return celery


def create_app(settings_override=None):
    app = connexion.App(__name__, specification_dir="./spec/", options={"swagger_ui": False}, debug=settings.DEBUG)
    app.add_api("openapi.yaml", arguments={"title": "AFFO Email Service API"})
    app.app.register_error_handler(connexion_buzz.ConnexionBuzz, connexion_buzz.ConnexionBuzz.build_error_handler())

    application = app.app
    application.config.from_object(settings)

    if settings_override:
        application.config.update(settings_override)

    # Import DB models. Flask-SQLAlchemy doesn't do this automatically.
    with application.app_context():
        for module in application.config.get("SQLALCHEMY_MODEL_IMPORTS", list()):
            importlib.import_module(module)

    # Initialize extensions/add-ons/plugins.
    db.init_app(application)
    ma.init_app(application)
    mail.init_app(application)
    migrate.init_app(application, db)
    routes.init_app(application)

    sentry_sdk.init(
        integrations=[CeleryIntegration(), FlaskIntegration()],
        dsn=application.config.get("SENTRY_DSN"),
        environment=application.config.get("ENV"),
        release=VERSION,
    )

    return application



import flask

import itsdangerous

from affo_email_service import settings
from affo_email_service.extensions import db
from affo_email_service.models.unsubscribe import Unsubscribe


unsubscribe_bp = flask.Blueprint("unsubscribe", __name__, url_prefix="/unsubscribe")


@unsubscribe_bp.route("/<token>/")
def unsubscribe(token):
    safe = itsdangerous.URLSafeTimedSerializer(settings.SECRET_KEY)

    token_data = safe.loads(token)

    with db.session.begin(subtransactions=True):
        # Create an unsubscribe
        unsubscribe_ = Unsubscribe(**token_data)
        db.session.add(unsubscribe_)

    return flask.render_template("unsubscribe.html"), 200



import os

from celery import Celery

from flask_mail import Mail

from flask_marshmallow import Marshmallow

from flask_migrate import Migrate

from flask_sqlalchemy import SQLAlchemy


__all__ = ["db", "ma", "mail", "migrate", "celery"]

db = SQLAlchemy(session_options={"autocommit": True})

ma = Marshmallow()

mail = Mail()

migrate = Migrate(directory=os.path.join(os.path.dirname(os.path.abspath(__file__)), "migrations"))

celery = Celery()



MESSAGE_STATUS_NEW = "NEW"
MESSAGE_STATUS_PENDING = "PENDING"
MESSAGE_STATUS_FAILED = "FAILED"
MESSAGE_STATUS_SUCCEDED = "SUCCEDED"
MESSAGE_STATUS_SKIPPED = "SKIPPED"
MESSAGE_STATUS_PARTIALLY_SKIPPED = "PARTIALLY_SKIPPED"



import datetime

from affo_email_service.extensions import db


class Unsubscribe(db.Model):  # noqa
    id = db.Column(db.Integer(), primary_key=True)
    email = db.Column(db.String(256))
    tag = db.Column(db.String(64))
    created_at = db.Column(db.DateTime, default=datetime.datetime.utcnow)



"""empty message

Revision ID: b96e7daa1cdb
Revises:
Create Date: 2019-05-17 07:14:08.566157

"""
from alembic import op
import sqlalchemy as sa

import sqlalchemy_utils


# revision identifiers, used by Alembic.
revision = 'b96e7daa1cdb'
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('message',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('from_', sa.String(length=256), nullable=True),
    sa.Column('to', sqlalchemy_utils.types.scalar_list.ScalarListType(), nullable=True),
    sa.Column('cc', sqlalchemy_utils.types.scalar_list.ScalarListType(), nullable=True),
    sa.Column('bcc', sqlalchemy_utils.types.scalar_list.ScalarListType(), nullable=True),
    sa.Column('subject', sa.String(length=256), nullable=True),
    sa.Column('text', sa.Text(), nullable=True),
    sa.Column('html', sa.Text(), nullable=True),
    sa.Column('attachment', sa.Boolean(), nullable=True),
    sa.Column('tag', sa.String(length=64), nullable=True),
    sa.Column('status', sa.Enum('NEW', 'PENDING', 'FAILED', 'SUCCEDED', 'SKIPPED', 'PARTIALLY_SKIPPED'), nullable=True),
    sa.Column('updated_at', sa.DateTime(), nullable=True),
    sa.Column('created_at', sa.DateTime(), nullable=True),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('unsubscribe',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('email', sa.String(length=256), nullable=True),
    sa.Column('tag', sa.String(length=64), nullable=True),
    sa.Column('created_at', sa.DateTime(), nullable=True),
    sa.PrimaryKeyConstraint('id')
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('unsubscribe')
    op.drop_table('message')
    # ### end Alembic commands ###



from .feedback import feedback_bp
from .template import template_bp
from .unsubscribe import unsubscribe_bp


def init_app(app):
    app.register_blueprint(feedback_bp)
    app.register_blueprint(template_bp)
    app.register_blueprint(unsubscribe_bp)



import ast
import codecs
import os
import re

from setuptools import find_packages, setup

here = os.path.abspath(os.path.dirname(__file__))

# Extra commands for documentation management
cmdclass = {}
command_options = {}

# Build Sphinx documentation (html)
# python setup.py build_sphinx
# generates files into build/sphinx/html
try:
    from sphinx.setup_command import BuildDoc

    cmdclass["build_sphinx"] = BuildDoc
except ImportError:
    pass


# Upload Sphinx documentation to PyPI (using Sphinx-PyPI-upload)
# python setup.py build_sphinx
# updates documentation at http://packages.python.org/affo-email-service/
try:
    from sphinx_pypi_upload import UploadDoc

    cmdclass["upload_sphinx"] = UploadDoc
except ImportError:
    pass


# Get the long description
with codecs.open(os.path.join(here, "README.rst"), encoding="utf-8") as f:
    long_description = f.read()


# Get version
_version_re = re.compile(r"VERSION\s+=\s+(.*)")

with open("affo_email_service/__init__.py", "rb") as f:
    version = str(ast.literal_eval(_version_re.search(f.read().decode("utf-8")).group(1)))


# Get requirements
def get_requirements(env=None):
    requirements_filename = "requirements.txt"

    if env:
        requirements_filename = f"requirements-{env}.txt"

    with open(requirements_filename) as fp:
        return [x.strip() for x in fp.read().split("\n") if not x.startswith("#")]


install_requirements = get_requirements()
test_requirements = get_requirements("test")

setup(
    name="affo-email-service",
    version=version,
    description="",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    url="https://github.com/affoio/affo-email-service",
    author="Aleksey Dalekin",
    author_email="ald@affo.io",
    license="BSD",
    platforms=["any"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: BSD License",
        "Topic :: System :: Distributed Computing",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
        "Operating System :: OS Independent",
    ],
    keywords="email service",
    packages=find_packages(exclude=["test*"]),
    cmdclass=cmdclass,
    command_options=command_options,
    setup_requires=["pytest-runner"],
    install_requires=install_requirements,
    tests_require=test_requirements,
    include_package_data=True,
    entry_points={"console_scripts": ["affo-email-service = affo_email_service.cli:manager.run"]},
)



from affo_email_service.models.message import message_schema
from affo_email_service.services import email_service


def create(message):
    # FIXME: attachments aren't supported

    return message_schema.jsonify(email_service.send(**message)), 201



import smtplib

import flask_mail

from sqlalchemy.orm.exc import NoResultFound

from affo_email_service.extensions import celery, db, mail  # noqa
from affo_email_service.models.message import Message


@celery.task(autoretry_for=(smtplib.SMTPException, NoResultFound), retry_backoff=True)
def send_email(message_id):
    message = db.session.query(Message).filter(Message.id == message_id).one()

    email = flask_mail.Message(
        recipients=message.to,
        sender=message.from_,
        body=message.text,
        subject=message.subject,
        cc=message.cc,
        bcc=message.bcc,
        html=message.html,
    )

    # FIXME: attachments aren't supported
    mail.send(email)



import json

import flask

import requests

feedback_bp = flask.Blueprint("feedback", __name__, url_prefix="/feedback")


@feedback_bp.route("/ses/", methods=["POST", "GET"])
def ses():
    # AWS sends JSON with text/plain mimetype
    try:
        feedback_data = json.loads(flask.request.data)
    except ValueError:
        pass

    sns_message_type = flask.request.headers.get("X-Amz-Sns-Message-Type")

    # Subscribe to the SNS topic
    if sns_message_type == "SubscriptionConfirmation" and "SubscribeURL" in feedback_data:
        requests.get(feedback_data["SubscribeURL"])

    # TODO: handle bounces and complaints properly

    return "", 200
